#include <math.h>
#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr

void fourier(float data[], unsigned long nn, int isign)
{
	unsigned long n,mmax,m,j,istep,i;
	float wtemp,wr,wpr,wpi,wi,theta; //single precision for the trigonometric	recurrences. 
	float tempr,tempi;
	
	n=nn << 1;
	j=1;
	for (i=1;i<n;i+=2) 
	{ 
		//This is the bit-reversal section of the routine. 
		if (j > i) {
			SWAP(data[j],data[i]); //Exchange the two complex numbers.
			SWAP(data[j+1],data[i+1]);
		}
		m=nn;
		while (m >= 2 && j > m) {
			j -= m;
			m >>= 1;
		}
		j += m;
	} // reversing loop
	
	// Butterfly computations 
	mmax=2;
	while (n > mmax) { // Outer loop executed log2 nn times.
		istep=mmax << 1;
		theta=(float) isign*(6.28318530717959/mmax); // Initialize the trigonometric recurrence.
		wtemp=sin(0.5*theta);
		wpr = -2.0*wtemp*wtemp;
		wpi=sin(theta);
		wr=1.0;
		wi=0.0;
		for (m=1;m<mmax;m+=2) {//  Here are the two nested inner loops.
			for (i=m;i<=n;i+=istep) {
				j=i+mmax;   
				// This is the butterfly computation itself 
				tempr=wr*data[j]-wi*data[j+1];
				tempi=wr*data[j+1]+wi*data[j];
				data[j]=data[i]-tempr;
				data[j+1]=data[i+1]-tempi;
				data[i] += tempr;
				data[i+1] += tempi;
			}
			wr=(wtemp=wr)*wpr-wi*wpi+wr; //Trigonometric recurrence.
			wi=wi*wpr+wtemp*wpi+wi;
		}
		mmax=istep;
	}
	
}// end of fft function


void ifft(float data[], unsigned long nn)
{
	int i;
    
    fourier(data-1, nn, -1);
    // normalizing by 1/N
	for (i=0; i<2*nn; i++)
			data[i]=data[i]/(nn);
}

void fft(float data[], unsigned long nn)
{
	fourier(data-1, nn, 1);	
}


void block_float_cmplx_multiply( float dst[], float src[], int blksz)
{
  int i;
  float Real=0, Imag=0;
  
  for( i=0; i<blksz; i++)
  {
    Real = dst[2*i]*src[2*i]   - dst[2*i+1]*src[2*i+1]; // real part
    Imag = dst[2*i]*src[2*i+1] + dst[2*i+1]*src[2*i];   // imaginary part
    dst[2*i]   =Real;
    dst[2*i+1] =Imag; 
  }
}

void block_cmplx_zeropad( float dst[], int FROM, int TO)
{
  int i;
  for( i=FROM; i<TO; i++)
  {
    dst[2*i]   =0;
    dst[2*i+1] =0; 
  }
}
